// ==============================================================
// $Id: progress.cpp,v 1.2 2004/10/27 13:36:50 dh219 Exp $
// Proton-K by D Henderson orbiter@aibs.org.uk
// Copyright 2004
// Released under the terms of the LGPL: http://www.gnu.org/licenses/lgpl.txt

/*
    This software and sourcecode is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

// ==============================================================

#include "progress.h"

PARTICLESTREAMSPEC rcs_exhaust = {
0, 0.05, 3000, 5, 0.1, 0.1, 1, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
PARTICLESTREAMSPEC::LVL_PLIN, 0, 2,
PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};
PARTICLESTREAMSPEC oms_exhaust = {
0, 0.4, 3000, 150, 0.05, 0.5, 5, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
PARTICLESTREAMSPEC::LVL_PLIN, 0, 2,
PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};
PARTICLESTREAMSPEC oms_exhaust2 = {
0, 0.4, 3000, 50, 0.05, 0.1, 5.0/3.0, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
PARTICLESTREAMSPEC::LVL_PLIN, 0, 2,
PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};

/*PARTICLESTREAMSPEC rcs_exhaust = {
0, 0.01, 1000, 2, 0, 0.05, 1, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};
PARTICLESTREAMSPEC oms_exhaust = {
0, 0.1, 1000, 10, 0, 0.1, 1, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};
*/

PROGRESS::PROGRESS (OBJHANDLE hObj, int fmodel) : COMMON_VESSEL_EX( hObj )
{

	ph_first = NULL;

	hprogress = oapiLoadMesh("progressm");

	VZERO = _V(0,0,0);

	status = 0;

	comshift = _V(0,0,0);

	anim3 = CreateAnimation(0);
	
	int i;
	for( i = 0 ; i < 20 ; i++ ) {
		animcomp3[i] = NULL;
	}

	panels = 0.0;
	panelsdeploy = false;

};

void PROGRESS::SetClassCaps(  FILEHANDLE cfg ) {

	if( ph_first == NULL ) {
		ph_first = CreatePropellantResource(FUEL);
	}
	SetDefaultPropellantResource(ph_first);

	dh = CreateDock(_V(0,0,4.0),_V(0,0,1),_V(0,1,0));

}

void PROGRESS::TimeStep(  double simt ) {
	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ;
		Refresh();
	}

	if( panelsdeploy && animcomp3[0] != NULL && panels < 1.0 ) {
		panels += oapiGetSimStep() / 30.0;
		SetAnimation( anim3, panels );
	}
	else if( !panelsdeploy && panels > 0.01 ) {
		SetAnimation( anim3, panels );
	}

	if( GetDockStatus(dh) && panels < 0.999 ) {
		Undock(0);
	}

	if( 0.5 * GetAtmDensity() * pow( GetAirspeed(), 3.0 ) > 8e7 ) {

		SetEnableFocus(false);
		OBJHANDLE focus, us;
		focus = oapiGetFocusObject();
		us = GetHandle();
		if( focus == us ) {
			if( oapiGetVesselCount() > 1 ) {
				int i;
				for( i = 0 ; i< oapiGetVesselCount() ; i++ ) {
					if( oapiGetVesselByIndex(i) != us ) {
						oapiSetFocusObject( oapiGetVesselByIndex(i) );
						break;
					}
				}
			}
		}
		focus = oapiGetFocusObject();
		if( focus != us ) {
			oapiDeleteVessel( us );
			return;
		}
	}


}

void PROGRESS::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;

	ClearMeshesEx();
	while (oapiReadScenario_nextline (scn, line)) 
	{
		ParsePayloadString( line );

		if( !strnicmp( line, "CONFIGURATION", 13 ) ) {
			status = atoi( line+13 );
		}
		if( !strnicmp( line, "PANELS", 6 ) ) {
			panels = atof( line+6 );
		}
		else 
		{
			ParseScenarioLineEx (line, vs);
		}
	}

	Refresh();
}

void PROGRESS::SaveState( FILEHANDLE scn )
{
	SaveDefaultState (scn);
	oapiWriteScenario_int (scn, "CONFIGURATION", status );
	oapiWriteScenario_float (scn, "PANELS", panels );
	SaveDefaultStateEx(scn);
}

void PROGRESS::VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ) {
	Refresh();
	SetAnimState(anim3,panels);
}

void PROGRESS::Refresh() {
	switch( status ) {
	default:
		status = 0;
		SetMainStage();
	}
}

void PROGRESS::SetMainStage() {
	
	double lev;
	lev = GetThrusterGroupLevel( THGROUP_MAIN );

	ShiftCentreOfMass(VZERO-comshift);
	comshift = _V(0,0,0);
	ShiftCentreOfMass(comshift);

	SetCameraOffset( _V( 0.0, -1.55, 0.5)-comshift );

	
	ClearExhaustRefs();
	ClearAttExhaustRefs();
  	ClearThrusterDefinitions();
	ClearMeshesEx();	

	SetSize (15);
	SetEmptyMass ( MASS + CalculateMass() );
	
	SetTouchdownPoints (_V(0,-2,4)-comshift,_V(2,-2,-4)-comshift,_V(-2,-2,-4)-comshift);

	SetPMI (_V(5,5,0.85));
	SetCrossSections (_V(15,32,6));
	SetCW (1.1, 2.2, 3.5, 3.5);

	SetRotDrag (_V(0.1,0.1,0.01));
	SetPitchMomentScale (-3e-6);
	SetBankMomentScale (3e-6);
	SetLiftCoeffFunc(0); 


	if( animcomp3[0] != NULL ) {
		int i;
		for( i = 0 ; i < 20 ; i++ ) {
			DelAnimationComponent(anim3, animcomp3[i] );
			animcomp3[i] = NULL;
		}
	}

	
	VECTOR3 off;
	off = (VZERO-comshift);
	UINT msh;
	msh = AddMesh(hprogress,&off);
	SetMeshVisibilityMode(msh,MESHVIS_ALWAYS);

	DefineAnimations(msh);
		
	MeshVisibilityPreference(MESHVIS_ALWAYS);
	Draw(VZERO-comshift);

	th_first = CreateThruster (_V(0,0,2.6)-comshift, _V(0,0,1),  3924.0, ph_first, 2992.0);
	AddExhaustStream(th_first, &oms_exhaust);
	AddExhaustStream(th_first, &oms_exhaust2);


	th_pup[0] = CreateThruster (_V(-0.1, 1.38, -3.05)-comshift, _V(0,-1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pup[1] = CreateThruster (_V(0.1, 1.38, -3.05)-comshift, _V(0,-1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pup[2] = CreateThruster (_V(-1.12, -0.25, 0)-comshift, _V(0,1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pup[3] = CreateThruster (_V(1.12, -0.25, 0)-comshift, _V(0,1,0), 140.0, ph_first, 2903.0, 0.0);
	AddExhaustStream(th_pup[0], &rcs_exhaust);
	AddExhaustStream(th_pup[1], &rcs_exhaust);
	AddExhaustStream(th_pup[2], &rcs_exhaust);
	AddExhaustStream(th_pup[3], &rcs_exhaust);

	th_pdown[0] = CreateThruster (_V(-0.1, -1.38, -3.05)-comshift, _V(0,1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pdown[1] = CreateThruster (_V(0.1, -1.38, -3.05)-comshift, _V(0,1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pdown[2] = CreateThruster (_V(-1.12, 0.25, 0)-comshift, _V(0,-1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pdown[3] = CreateThruster (_V(1.12, 0.25, 0)-comshift, _V(0,-1,0), 140.0, ph_first, 2903.0, 0.0);

	AddExhaustStream(th_pdown[0], &rcs_exhaust);
	AddExhaustStream(th_pdown[1], &rcs_exhaust);
	AddExhaustStream(th_pdown[2], &rcs_exhaust);
	AddExhaustStream(th_pdown[3], &rcs_exhaust);


	th_rleft[0] = th_pup[3];
	th_rleft[1] = th_pdown[2];

	th_rright[0] = th_pup[2];
	th_rright[1] = th_pdown[3];


	th_yleft[0] = CreateThruster (_V(-1.38, -0.1, -3.05)-comshift, _V(1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yleft[1] = CreateThruster (_V(-1.38,  0.1, -3.05)-comshift, _V(1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yleft[2] = CreateThruster (_V(1.2, -0.1, 0)-comshift, _V(-1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yleft[3] = CreateThruster (_V(1.2,  0.1, 0)-comshift, _V(-1,0,0), 140.0, ph_first, 2903.0, 0.0);
	AddExhaustStream(th_yleft[0], &rcs_exhaust);
	AddExhaustStream(th_yleft[1], &rcs_exhaust);
	AddExhaustStream(th_yleft[2], &rcs_exhaust);
	AddExhaustStream(th_yleft[3], &rcs_exhaust);
	

	th_yright[0] = CreateThruster (_V(1.38, -0.1, -3.05)-comshift, _V(-1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yright[1] = CreateThruster (_V(1.38,  0.1, -3.05)-comshift, _V(-1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yright[2] = CreateThruster (_V(-1.2, -0.1, 0)-comshift, _V(1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yright[3] = CreateThruster (_V(-1.2,  0.1, 0)-comshift, _V(1,0,0), 140.0, ph_first, 2903.0, 0.0);

	AddExhaustStream(th_yright[0], &rcs_exhaust);
	AddExhaustStream(th_yright[1], &rcs_exhaust);
	AddExhaustStream(th_yright[2], &rcs_exhaust);
	AddExhaustStream(th_yright[3], &rcs_exhaust);

	th_left[0] = th_yleft[2];
	th_left[1] = th_yleft[3];

	th_right[0] = th_yright[2];
	th_right[1] = th_yright[3];
	
	th_up[0] = th_pup[2];
	th_up[1] = th_pup[3];

	th_down[0] = th_pdown[2];
	th_down[1] = th_pdown[3];




	th_fore[0] = CreateThruster (_V(-1.1, 0, -3.1)-comshift, _V(0,0,1), 140.0, ph_first, 2903.0, 0.0);
	th_fore[1] = CreateThruster (_V(1.1, 0, -3.1)-comshift, _V(0,0,1), 140.0, ph_first, 2903.0, 0.0);
	th_fore[2] = CreateThruster (_V(0, 1.1, -3.1)-comshift, _V(0,0,1), 140.0, ph_first, 2903.0, 0.0);
	th_fore[3] = CreateThruster (_V(0, -1.1, -3.1)-comshift, _V(0,0,1), 140.0, ph_first, 2903.0, 0.0);
	AddExhaustStream(th_fore[0], &rcs_exhaust);
	AddExhaustStream(th_fore[1], &rcs_exhaust);
	AddExhaustStream(th_fore[2], &rcs_exhaust);
	AddExhaustStream(th_fore[3], &rcs_exhaust);


	th_aft[0] = CreateThruster (_V(-1.15, 0, 0.04)-comshift, _V(0.141,0,-0.99), 140.0, ph_first, 2903.0, 0.0);
	th_aft[1] = CreateThruster (_V(1.15, 0, 0.04)-comshift, _V(-0.141,0,-0.99), 140.0, ph_first, 2903.0, 0.0);
	AddExhaustStream(th_aft[0], &rcs_exhaust);
	AddExhaustStream(th_aft[1], &rcs_exhaust);


	CreateThrusterGroup( th_pup, 4, THGROUP_ATT_PITCHUP );
	CreateThrusterGroup( th_pdown, 4, THGROUP_ATT_PITCHDOWN );
	CreateThrusterGroup( th_yleft, 4, THGROUP_ATT_YAWLEFT );
	CreateThrusterGroup( th_yright, 4, THGROUP_ATT_YAWRIGHT );
	CreateThrusterGroup( th_rleft, 2, THGROUP_ATT_BANKLEFT );
	CreateThrusterGroup( th_rright, 2, THGROUP_ATT_BANKRIGHT );
	
	CreateThrusterGroup( th_left, 2, THGROUP_ATT_LEFT );
	CreateThrusterGroup( th_right, 2, THGROUP_ATT_RIGHT );
	CreateThrusterGroup( th_down, 2, THGROUP_ATT_DOWN );
	CreateThrusterGroup( th_up, 2, THGROUP_ATT_UP );
	CreateThrusterGroup( th_fore, 4, THGROUP_ATT_FORWARD );
	CreateThrusterGroup( th_aft, 2, THGROUP_ATT_BACK );

	CreateThrusterGroup( &th_first, 1, THGROUP_MAIN );
	CreateThrusterGroup( th_aft, 2, THGROUP_RETRO );


	SetAnimState(anim3,panels);


}


int PROGRESS::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate) {

		if (KEYDOWN (keystate, OAPI_KEY_K) && !KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) ) 
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_K, 1.0)) {
				panelsdeploy = true;
				return 1;
			}
		}


		if (KEYDOWN (keystate, OAPI_KEY_J) && !KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) ) 
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
				DeployPayload();
				Refresh();
				return 1;
			}
		}

	return 0;
}

void PROGRESS::DefineAnimations(UINT mshid) {

	MGROUP_ROTATE *mgroup;
	MGROUP_TRANSLATE *mgroupt;

	/*** left panel */
	
	static UINT groups0[1] = { 16 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups0, 1,
			_V(-1.1,0,0),
			_V(0,0,1),
			90*RAD
		);
	animcomp3[0] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup);
	
	static UINT groups1[1] = { 15 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups1, 1,
			_V(-1.12,0.63,0),
			_V(0,0,1),
			-178*RAD
		);
	animcomp3[1] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup, animcomp3[0]);

	static UINT groups2[1] = { 20 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups2, 1,
			_V(-1.15,-0.63,0),
			_V(0,0,1),
			178*RAD
		);
	animcomp3[2] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup, animcomp3[1]);

	static UINT groups3[1] = { 19 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups3, 1,
			_V(-1.18,0.63,0),
			_V(0,0,1),
			-178*RAD
		);
	animcomp3[3] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup, animcomp3[2]);


	/*** right panel */
	
	static UINT groups4[1] = { 21 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups4, 1,
			_V(1.1,0,0),
			_V(0,0,1),
			-90*RAD
		);
	animcomp3[4] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup);
	
	static UINT groups5[1] = { 22 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups5, 1,
			_V(1.12,0.63,0),
			_V(0,0,1),
			178*RAD
		);
	animcomp3[5] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup, animcomp3[4]);

	static UINT groups6[1] = { 24 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups6, 1,
			_V(1.15,-0.63,0),
			_V(0,0,1),
			-178*RAD
		);
	animcomp3[6] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup, animcomp3[5]);

	static UINT groups7[] = { 23 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups7, 1,
			_V(1.18,0.63,0),
			_V(0,0,1),
			178*RAD
		);
	animcomp3[7] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup, animcomp3[6]);


	/* antennae */


	static UINT groups8[1] = { 18 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups8, 1,
			_V(-0.722,0.891,-3.1),
			_V(0.247,1,0),
			178*RAD
		);
	animcomp3[8] = AddAnimationComponent( anim3, 0.05, 0.3, mgroup);

	static UINT groups9[1] = { 6 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups9, 1,
			_V(0,0.94,3.61),
			_V(1,0,0),
			-140*RAD
		);
	animcomp3[9] = AddAnimationComponent( anim3, 0.31, 0.85, mgroup);

	static UINT groups10[1] = { 6 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups10, 1,
			_V(0,0.94,3.61),
			_V(1,0,0),
			-10*RAD
		);
	animcomp3[10] = AddAnimationComponent( anim3, 0.27, 0.3, mgroup);

	
		
	static UINT groups11[1] = { 7 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups11, 1,
			_V(-0.94,0,3.61),
			_V(0,1,0),
			-130*RAD
		);
	animcomp3[11] = AddAnimationComponent( anim3, 0.3, 0.9, mgroup);

	static UINT groups12[1] = { 7 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups12, 1,
			_V(-0.94,0,3.61),
			_V(0,1,0),
			-20*RAD
		);
	animcomp3[12] = AddAnimationComponent( anim3, 0.3, 0.35, mgroup);

	static UINT groups13[3] = { 26,27,28 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups13, 3,
			_V(0.057,-1.167,0),
			_V(0,0,1),
			-95*RAD
		);
	animcomp3[13] = AddAnimationComponent( anim3, 0.3, 0.35, mgroup);



	/* thingie */

	static UINT groups14[1] = { 3 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups14, 1,
			_V(0.565,-0.35,3.86),
			_V(0.574,1,0),
			-50*RAD
		);
	animcomp3[14] = AddAnimationComponent( anim3, 0.02, 0.1, mgroup);

	static UINT groups15[1] = { 5 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups15, 1,
			_V(0.794,-0.484,3.593),
			_V(0.574,1,0),
			50*RAD
		);
	animcomp3[15] = AddAnimationComponent( anim3, 0.02, 0.1, mgroup, animcomp3[14]);



}
