// ==============================================================
// $Id: progress.h,v 1.1 2004/10/21 22:09:48 dh219 Exp $
// Soyuz PROGRESS by D Henderson orbiter@aibs.org.uk
// Copyright 2004
// Released under the terms of the LGPL: http://www.gnu.org/licenses/lgpl.txt

/*
    This software and sourcecode is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

// ==============================================================

#include "cvel.h"
#include "orbitersdk.h"
#include <stdio.h>
#include <math.h>

#define MASS 6550.0 
#define MASSENTRY 2950.0

#define FUEL 900.0

class PROGRESS: public COMMON_VESSEL_EX {
public:
	PROGRESS(OBJHANDLE hObj, int fmodel);
	void TimeStep(  double simt );
	void SetMainStage ();
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState( FILEHANDLE scn );
	void SetClassCaps(  FILEHANDLE cfg );
	int ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);
	void Separate();
	void Refresh();
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount );
	void DefineAnimations(UINT meshid);

protected:
	void AddThrusters();
	PROPELLANT_HANDLE ph_first;	
	THRUSTER_HANDLE th_first;
	THRUSTER_HANDLE th_pup[4];
	THRUSTER_HANDLE th_pdown[4];
	THRUSTER_HANDLE th_yleft[4];
	THRUSTER_HANDLE th_yright[4];
	THRUSTER_HANDLE th_rleft[4];
	THRUSTER_HANDLE th_rright[4];
	THRUSTER_HANDLE th_up[4];
	THRUSTER_HANDLE th_down[4];
	THRUSTER_HANDLE th_left[4];
	THRUSTER_HANDLE th_right[4];
	THRUSTER_HANDLE th_fore[4];
	THRUSTER_HANDLE th_aft[4];


	MESHHANDLE hprogress;
	
	SURFHANDLE tex;

	VECTOR3 VZERO;

	short status;

	VECTOR3 comshift;
	
	short anim3;
	ANIMATIONCOMPONENT_HANDLE animcomp3[20];

	float panels;
	bool panelsdeploy;

	DOCKHANDLE dh;
};

/**
*
*
*/
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	PROGRESS *progress= new PROGRESS( hvessel, flightmodel );
	return progress;
}

/**
*
*
*/
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (PROGRESS*)vessel;
}



/**
*
*
*/
DLLCLBK int ovcConsumeBufferedKey (VESSEL *vessel, DWORD dwKey, bool fPressed, const char *keystate)
{
	PROGRESS *progress = (PROGRESS*)vessel;
	return progress->ConsumeBufferedKey( dwKey, fPressed, keystate );
}
/**
*
*
*/
DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	PROGRESS *progress = (PROGRESS*)vessel;
	progress->SetClassCaps(  cfg );

}

/**
*
*
*/
DLLCLBK void ovcTimestep (VESSEL *vessel, double simt)
{
	PROGRESS *progress = (PROGRESS*)vessel;
	progress->TimeStep(  simt );
}
/**
*
*
*/
DLLCLBK void ovcVisualCreated ( VESSEL *vessel, VISHANDLE vis, int refcount) {

	PROGRESS *progress = (PROGRESS*)vessel;
	progress->VisualCreated ( vessel,  vis,  refcount);
}

/**
*
*
*/
DLLCLBK void ovcLoadStateEx (VESSEL *vessel, FILEHANDLE scn, void *vs)

{
	PROGRESS *progress = (PROGRESS*)vessel;
	progress->LoadState(  scn, vs );
}


/**
*
*
*/
DLLCLBK void ovcSaveState (VESSEL *vessel, FILEHANDLE scn)
{
	PROGRESS *progress= (PROGRESS*)vessel;
	progress->SaveState(  scn );
}


